var util = require('util');
var path = require('path');
var yeoman = require('yeoman-generator');


module.exports = Generator;

function Generator(args, options, config) {
  yeoman.generators.Base.apply(this, arguments);

  this.testFramework = this.options['test-framework'] || 'mocha';
  this.templateFramework = 'handlebars';
  this.hookFor(this.testFramework, { as: 'app' });

  this.on('end', function () {
    if (['app', 'backbone', 'marionette'].indexOf(this.generatorName) >= 0) {
      this.installDependencies({ skipInstall: this.options['skip-install'] });
    }
  });
}

util.inherits(Generator, yeoman.generators.Base);

Generator.prototype.askFor = function askFor() {
  var cb = this.async();

  // welcome message
  var welcome =
  '\n     _-----_' +
  '\n    |       |' +
  '\n    |'+'--(o)--'.red+'|   .--------------------------.' +
  '\n   `---------´  |    '+'Welcome to Yeoman,'.yellow.bold+'    |' +
  '\n    '+'( '.yellow+'_'+'´U`'.yellow+'_'+' )'.yellow+'   |   '+'ladies and gentlemen!'.yellow.bold+'  |' +
  '\n    /___A___\\   \'__________________________\'' +
  '\n     |  ~  |'.yellow +
  '\n   __'+'\'.___.\''.yellow+'__' +
  '\n ´   '+'`  |'.red+'° '+'´ Y'.red+' `\n';

  console.log(welcome);
  console.log('Out of the box I include HTML5 Boilerplate, jQuery, Backbone.js, Marionette, Require and Modernizr.');


  var prompts = [{
    name: 'useMongoose',
    message: 'Would you like to include MongoDB for storage?',
    default: 'Y/n',
    warning: 'Yes: Ok cool.'
  },
  {
    name: 'useSocketIO',
    message: 'Would you like to include Socket IO for real time communication?',
    default: 'Y/n',
    warning: 'Yes: Ok cool.'
  },
  {
    name: 'useFaye',
    message: 'Would you like to include Faye for real time communication?',
    default: 'Y/n',
    warning: 'Yes: Ok cool.'
  },
  {
    name: 'useBaucis',
    message: 'Would you like to include Baucis for REST?',
    default: 'Y/n',
    warning: 'Yes: Ok cool.'
  }];

  this.prompt(prompts, function (err, props) {
    if (err) {
      return this.emit('error', err);
    }

    // manually deal with the response, get back and store the results.
    // we change a bit this way of doing to automatically do this in the self.prompt() method.
    this.useMongoose = (/y/i).test(props.useMongoose);
    this.useSocketIO = (/y/i).test(props.useSocketIO);
    this.useFaye = (/y/i).test(props.useFaye);
    this.useBaucis = (/y/i).test(props.useBaucis);

    //dummy vars for legacy
    this.compassBootstrap = true;
    this.includeRequireJS = true;

    cb();
  }.bind(this));
};

Generator.prototype.git = function git() {
  this.copy('gitignore', '.gitignore');
  this.copy('gitattributes', '.gitattributes');
};

Generator.prototype.bower = function bower() {
  this.copy('bowerrc', '.bowerrc');
  this.copy('_bower.json', 'bower.json');
};

Generator.prototype.jshint = function jshint() {
  this.copy('jshintrc', '.jshintrc');
};

Generator.prototype.editorConfig = function editorConfig() {
  this.copy('editorconfig', '.editorconfig');
};

Generator.prototype.gruntfile = function gruntfile() {
  if (this.testFramework === 'jasmine') {
    this.write('Gruntfile.js', this.engine(this.read('Gruntfile.js'), this).replace(/mocha/g, 'jasmine'));
  } else {
    this.template('Gruntfile.js');
  }
};

Generator.prototype.packageJSON = function packageJSON() {
  this.template('_package.json', 'package.json');
};

Generator.prototype.mainStylesheet = function mainStylesheet() {
  if (this.compassBootstrap) {
    this.write('app/styles/main.scss', '@import \'sass-bootstrap/lib/bootstrap\';\n\n.hero-unit {\n    margin: 50px auto 0 auto;\n    width: 400px;\n}');
  } else {
    this.write('app/styles/main.css', 'body {\n    background: #fafafa;\n}\n\n.hero-unit {\n    margin: 50px auto 0 auto;\n    width: 300px;\n}');
  }
};


Generator.prototype.bootstrapJs = function bootstrapJs() {
  // TODO: create a Bower component for this
  if (this.includeRequireJS && this.compassBootstrap) {
    this.copy('bootstrap.js', 'app/scripts/vendor/bootstrap.js');
  }
};

Generator.prototype.setupEnv = function setupEnv() {
  this.mkdir('app/templates');

  this.mkdir('server');

  this.mkdir('app');
  this.mkdir('app/scripts');
  this.mkdir('app/scripts/vendor/');
  this.mkdir('app/styles');
  this.mkdir('app/images');
  this.template('app/404.html');
  this.template('app/favicon.ico');
  this.template('app/robots.txt');
  this.copy('app/htaccess', 'app/.htaccess');

  this.template('server/app.js', 'server/app.js');
  this.copy('server/mongooseInit.js', 'server/mongooseInit.js');

  this.copy( 'app/index.html', 'app/index.html' );
  this.copy( 'app/main.js', 'app/scripts/main.js' );
  this.copy( 'app/init.js', 'app/scripts/init.js' );
  this.copy( 'app/regionManager.js', 'app/scripts/regionManager.js' );
  this.copy( 'app/application.js', 'app/scripts/application.js' );
  this.copy( 'app/communicator.js', 'app/scripts/communicator.js' );

  this.copy( 'app/welcome.hbs', 'app/templates/welcome.hbs');
};

